#!/usr/bin/env ruby

require 'cgi'
require 'open-uri'
require 'kconv'
require 'rexml/document'

class String
	def to_degree
		s = dup
		sign = s.sub!( /^-/, '' ) ? -1 : 1
		a = s.split( /\// )
		sign * (a[0].to_f + a[1].to_f/60 + a[2].to_f/3600)
	end

	def to_json
		toutf16.unpack( "n*" ).map do |n|
			(n < 128 ? '%c' : '\u%04x') % n
		end.join
	end
end

class Coordinates
	attr_reader :lat, :lon

	def initialize( str )
		@lat, @lon = str.split( /,/ ).map{|i| i.to_degree}
	end

	def distance( pos )
		radius = 6370300.0
		x1 = Math.cos( lon*Math::PI/180.0 )*Math.cos( lat*Math::PI/180.0 )
		y1 = Math.sin( lon*Math::PI/180.0 )*Math.cos( lat*Math::PI/180.0 )
		z1 = Math.sin( lat*Math::PI/180.0 )
		x2 = Math.cos( pos.lon*Math::PI/180.0 )*Math.cos( pos.lat*Math::PI/180.0 )
		y2 = Math.sin( pos.lon*Math::PI/180.0 )*Math.cos( pos.lat*Math::PI/180.0 )
		z2 = Math.sin( pos.lat*Math::PI/180.0 )
		d = x1*x2+y1*y2+z1*z2
		d = -1 if d<-1
		d = 1 if d>1
		radius*Math.acos( d )
	end
end

cgi = CGI.new
pos = cgi.params['pos'][0]

if pos then
	uri = "http://api.alpslab.jp/v1/search"
	uri << "?appid=guest"
	uri << "&p=#{CGI::escape( pos )}"
	uri << "&r=1000&type=photo"
	doc = open( uri ){|f| REXML::Document.new( f.read ).root}
	if doc && doc.elements['item'] then
		photos = []
		p = Coordinates.new( pos )
		doc.elements.each( 'item' ) do |e|
			c = Coordinates.new( e.elements['coordinates'].text )
			d = c.distance( p )
			photos << [e.attributes['id'], d]
		end
		id = photos.min{|a, b| a.last <=> b.last}.first
	end
end

if id then
	uri = "http://api.alpslab.jp/v1/detail"
	uri << "?appid=guest"
	uri << "&id=#{CGI::escape( id )}"
	doc = open( uri ){|f| REXML::Document.new( f.read ).root}
	if doc && doc.elements['type'] && doc.elements['type'].text == 'photo' then
		r = {}
		r['title'] = doc.elements['title'].text
		r['link'] = doc.elements['link'].text
		r['coordinates'] = doc.elements['coordinates'].text
		r['image'] = doc.elements['small'].text
		r['comment'] = doc.elements['comment'].text
	end
end

if r then
	json = '{'
	json << r.map do |k, v|
		"#{k}:#{v.to_json.dump.gsub( /\\\\u/, '\u' )}"
	end.join( ',' )
	json << '}'
end

print cgi.header( 'type' => 'text/javascript' )
print "photo_callback(#{json});"

exit
